/*
qxgedit - Qt XG Editor.

   Copyright (C) 2005-2023, rncbc aka Rui Nuno Capela. All rights reserved.

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License along
   with this program; if not, write to the Free Software Foundation, Inc.,
   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

 
*/

/********************************************************************************
** Form generated from reading UI file 'qxgeditOptionsForm.ui'
**
** Created by: Qt User Interface Compiler version 6.8.3
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_QXGEDITOPTIONSFORM_H
#define UI_QXGEDITOPTIONSFORM_H

#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QDoubleSpinBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QListWidget>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QSpinBox>
#include <QtWidgets/QTabWidget>
#include <QtWidgets/QToolButton>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_qxgeditOptionsForm
{
public:
    QVBoxLayout *vboxLayout;
    QTabWidget *OptionsTabWidget;
    QWidget *GeneralTabPage;
    QGridLayout *gridLayout;
    QCheckBox *ConfirmResetCheckBox;
    QSpacerItem *spacerItem;
    QCheckBox *ConfirmRemoveCheckBox;
    QCheckBox *CompletePathCheckBox;
    QLabel *MaxRecentFilesTextLabel;
    QSpinBox *MaxRecentFilesSpinBox;
    QLabel *RandomizePercentTextLabel;
    QDoubleSpinBox *RandomizePercentSpinBox;
    QLabel *BaseFontSizeTextLabel;
    QComboBox *BaseFontSizeComboBox;
    QLabel *StyleThemeTextLabel;
    QComboBox *StyleThemeComboBox;
    QLabel *ColorThemeTextLabel;
    QComboBox *ColorThemeComboBox;
    QToolButton *ColorThemeToolButton;
    QSpacerItem *spacerItem1;
    QWidget *MidiTabPage;
    QVBoxLayout *vboxLayout1;
    QLabel *MidiInputLabel;
    QListWidget *MidiInputListView;
    QLabel *MidiOutputLabel;
    QListWidget *MidiOutputListView;
    QSpacerItem *spacerItem2;
    QDialogButtonBox *DialogButtonBox;

    void setupUi(QDialog *qxgeditOptionsForm)
    {
        if (qxgeditOptionsForm->objectName().isEmpty())
            qxgeditOptionsForm->setObjectName("qxgeditOptionsForm");
        qxgeditOptionsForm->resize(360, 320);
        const QIcon icon = QIcon(QString::fromUtf8(":/images/qxgedit.svg"));
        qxgeditOptionsForm->setWindowIcon(icon);
        qxgeditOptionsForm->setSizeGripEnabled(true);
        vboxLayout = new QVBoxLayout(qxgeditOptionsForm);
        vboxLayout->setSpacing(8);
        vboxLayout->setContentsMargins(8, 8, 8, 8);
        vboxLayout->setObjectName("vboxLayout");
        OptionsTabWidget = new QTabWidget(qxgeditOptionsForm);
        OptionsTabWidget->setObjectName("OptionsTabWidget");
        OptionsTabWidget->setAcceptDrops(false);
        GeneralTabPage = new QWidget();
        GeneralTabPage->setObjectName("GeneralTabPage");
        gridLayout = new QGridLayout(GeneralTabPage);
        gridLayout->setSpacing(8);
        gridLayout->setContentsMargins(8, 8, 8, 8);
        gridLayout->setObjectName("gridLayout");
        ConfirmResetCheckBox = new QCheckBox(GeneralTabPage);
        ConfirmResetCheckBox->setObjectName("ConfirmResetCheckBox");

        gridLayout->addWidget(ConfirmResetCheckBox, 0, 0, 1, 3);

        spacerItem = new QSpacerItem(8, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        gridLayout->addItem(spacerItem, 0, 3, 8, 1);

        ConfirmRemoveCheckBox = new QCheckBox(GeneralTabPage);
        ConfirmRemoveCheckBox->setObjectName("ConfirmRemoveCheckBox");

        gridLayout->addWidget(ConfirmRemoveCheckBox, 1, 0, 1, 3);

        CompletePathCheckBox = new QCheckBox(GeneralTabPage);
        CompletePathCheckBox->setObjectName("CompletePathCheckBox");

        gridLayout->addWidget(CompletePathCheckBox, 2, 0, 1, 3);

        MaxRecentFilesTextLabel = new QLabel(GeneralTabPage);
        MaxRecentFilesTextLabel->setObjectName("MaxRecentFilesTextLabel");

        gridLayout->addWidget(MaxRecentFilesTextLabel, 3, 0, 1, 1);

        MaxRecentFilesSpinBox = new QSpinBox(GeneralTabPage);
        MaxRecentFilesSpinBox->setObjectName("MaxRecentFilesSpinBox");
        MaxRecentFilesSpinBox->setMinimum(0);
        MaxRecentFilesSpinBox->setMaximum(20);
        MaxRecentFilesSpinBox->setValue(5);

        gridLayout->addWidget(MaxRecentFilesSpinBox, 3, 1, 1, 1);

        RandomizePercentTextLabel = new QLabel(GeneralTabPage);
        RandomizePercentTextLabel->setObjectName("RandomizePercentTextLabel");

        gridLayout->addWidget(RandomizePercentTextLabel, 4, 0, 1, 1);

        RandomizePercentSpinBox = new QDoubleSpinBox(GeneralTabPage);
        RandomizePercentSpinBox->setObjectName("RandomizePercentSpinBox");
        RandomizePercentSpinBox->setAccelerated(true);
        RandomizePercentSpinBox->setDecimals(1);
        RandomizePercentSpinBox->setMinimum(0.100000000000000);
        RandomizePercentSpinBox->setMaximum(100.000000000000000);
        RandomizePercentSpinBox->setSingleStep(0.100000000000000);
        RandomizePercentSpinBox->setValue(20.000000000000000);

        gridLayout->addWidget(RandomizePercentSpinBox, 4, 1, 1, 1);

        BaseFontSizeTextLabel = new QLabel(GeneralTabPage);
        BaseFontSizeTextLabel->setObjectName("BaseFontSizeTextLabel");

        gridLayout->addWidget(BaseFontSizeTextLabel, 5, 0, 1, 1);

        BaseFontSizeComboBox = new QComboBox(GeneralTabPage);
        BaseFontSizeComboBox->addItem(QString());
        BaseFontSizeComboBox->addItem(QString());
        BaseFontSizeComboBox->addItem(QString());
        BaseFontSizeComboBox->addItem(QString());
        BaseFontSizeComboBox->addItem(QString());
        BaseFontSizeComboBox->addItem(QString());
        BaseFontSizeComboBox->addItem(QString());
        BaseFontSizeComboBox->addItem(QString());
        BaseFontSizeComboBox->setObjectName("BaseFontSizeComboBox");
        BaseFontSizeComboBox->setEditable(true);

        gridLayout->addWidget(BaseFontSizeComboBox, 5, 1, 1, 1);

        StyleThemeTextLabel = new QLabel(GeneralTabPage);
        StyleThemeTextLabel->setObjectName("StyleThemeTextLabel");

        gridLayout->addWidget(StyleThemeTextLabel, 6, 0, 1, 1);

        StyleThemeComboBox = new QComboBox(GeneralTabPage);
        StyleThemeComboBox->addItem(QString());
        StyleThemeComboBox->setObjectName("StyleThemeComboBox");
        StyleThemeComboBox->setEditable(false);

        gridLayout->addWidget(StyleThemeComboBox, 6, 1, 1, 1);

        ColorThemeTextLabel = new QLabel(GeneralTabPage);
        ColorThemeTextLabel->setObjectName("ColorThemeTextLabel");

        gridLayout->addWidget(ColorThemeTextLabel, 7, 0, 1, 1);

        ColorThemeComboBox = new QComboBox(GeneralTabPage);
        ColorThemeComboBox->addItem(QString());
        ColorThemeComboBox->addItem(QString());
        ColorThemeComboBox->addItem(QString());
        ColorThemeComboBox->setObjectName("ColorThemeComboBox");
        ColorThemeComboBox->setEditable(false);

        gridLayout->addWidget(ColorThemeComboBox, 7, 1, 1, 1);

        ColorThemeToolButton = new QToolButton(GeneralTabPage);
        ColorThemeToolButton->setObjectName("ColorThemeToolButton");
        ColorThemeToolButton->setMinimumSize(QSize(22, 22));
        ColorThemeToolButton->setMaximumSize(QSize(24, 24));
        ColorThemeToolButton->setFocusPolicy(Qt::TabFocus);

        gridLayout->addWidget(ColorThemeToolButton, 7, 2, 1, 1);

        spacerItem1 = new QSpacerItem(20, 8, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        gridLayout->addItem(spacerItem1, 8, 0, 1, 4);

        OptionsTabWidget->addTab(GeneralTabPage, QString());
        MidiTabPage = new QWidget();
        MidiTabPage->setObjectName("MidiTabPage");
        vboxLayout1 = new QVBoxLayout(MidiTabPage);
        vboxLayout1->setSpacing(8);
        vboxLayout1->setContentsMargins(8, 8, 8, 8);
        vboxLayout1->setObjectName("vboxLayout1");
        MidiInputLabel = new QLabel(MidiTabPage);
        MidiInputLabel->setObjectName("MidiInputLabel");

        vboxLayout1->addWidget(MidiInputLabel);

        MidiInputListView = new QListWidget(MidiTabPage);
        MidiInputListView->setObjectName("MidiInputListView");
        MidiInputListView->setMinimumSize(QSize(320, 60));
        MidiInputListView->setSelectionMode(QAbstractItemView::ExtendedSelection);
        MidiInputListView->setUniformItemSizes(true);

        vboxLayout1->addWidget(MidiInputListView);

        MidiOutputLabel = new QLabel(MidiTabPage);
        MidiOutputLabel->setObjectName("MidiOutputLabel");

        vboxLayout1->addWidget(MidiOutputLabel);

        MidiOutputListView = new QListWidget(MidiTabPage);
        MidiOutputListView->setObjectName("MidiOutputListView");
        MidiOutputListView->setMinimumSize(QSize(320, 60));
        MidiOutputListView->setSelectionMode(QAbstractItemView::ExtendedSelection);
        MidiOutputListView->setUniformItemSizes(true);

        vboxLayout1->addWidget(MidiOutputListView);

        spacerItem2 = new QSpacerItem(20, 0, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        vboxLayout1->addItem(spacerItem2);

        OptionsTabWidget->addTab(MidiTabPage, QString());

        vboxLayout->addWidget(OptionsTabWidget);

        DialogButtonBox = new QDialogButtonBox(qxgeditOptionsForm);
        DialogButtonBox->setObjectName("DialogButtonBox");
        DialogButtonBox->setOrientation(Qt::Horizontal);
        DialogButtonBox->setStandardButtons(QDialogButtonBox::Cancel|QDialogButtonBox::Ok);

        vboxLayout->addWidget(DialogButtonBox);

#if QT_CONFIG(shortcut)
        MaxRecentFilesTextLabel->setBuddy(MaxRecentFilesSpinBox);
        RandomizePercentTextLabel->setBuddy(RandomizePercentSpinBox);
        BaseFontSizeTextLabel->setBuddy(BaseFontSizeComboBox);
        StyleThemeTextLabel->setBuddy(StyleThemeComboBox);
        ColorThemeTextLabel->setBuddy(ColorThemeComboBox);
        MidiInputLabel->setBuddy(MidiInputListView);
        MidiOutputLabel->setBuddy(MidiOutputListView);
#endif // QT_CONFIG(shortcut)
        QWidget::setTabOrder(OptionsTabWidget, ConfirmResetCheckBox);
        QWidget::setTabOrder(ConfirmResetCheckBox, ConfirmRemoveCheckBox);
        QWidget::setTabOrder(ConfirmRemoveCheckBox, CompletePathCheckBox);
        QWidget::setTabOrder(CompletePathCheckBox, MaxRecentFilesSpinBox);
        QWidget::setTabOrder(MaxRecentFilesSpinBox, RandomizePercentSpinBox);
        QWidget::setTabOrder(RandomizePercentSpinBox, BaseFontSizeComboBox);
        QWidget::setTabOrder(BaseFontSizeComboBox, StyleThemeComboBox);
        QWidget::setTabOrder(StyleThemeComboBox, ColorThemeComboBox);
        QWidget::setTabOrder(ColorThemeComboBox, MidiInputListView);
        QWidget::setTabOrder(MidiInputListView, MidiOutputListView);
        QWidget::setTabOrder(MidiOutputListView, ColorThemeToolButton);
        QWidget::setTabOrder(ColorThemeToolButton, DialogButtonBox);

        retranslateUi(qxgeditOptionsForm);

        OptionsTabWidget->setCurrentIndex(0);


        QMetaObject::connectSlotsByName(qxgeditOptionsForm);
    } // setupUi

    void retranslateUi(QDialog *qxgeditOptionsForm)
    {
        qxgeditOptionsForm->setWindowTitle(QCoreApplication::translate("qxgeditOptionsForm", "Options", nullptr));
#if QT_CONFIG(tooltip)
        ConfirmResetCheckBox->setToolTip(QCoreApplication::translate("qxgeditOptionsForm", "Whether to ask for confirmation on parameter reset", nullptr));
#endif // QT_CONFIG(tooltip)
        ConfirmResetCheckBox->setText(QCoreApplication::translate("qxgeditOptionsForm", "Confirm parameter &resets", nullptr));
#if QT_CONFIG(tooltip)
        ConfirmRemoveCheckBox->setToolTip(QCoreApplication::translate("qxgeditOptionsForm", "Whether to ask for confirmation on removals", nullptr));
#endif // QT_CONFIG(tooltip)
        ConfirmRemoveCheckBox->setText(QCoreApplication::translate("qxgeditOptionsForm", "&Confirm removals", nullptr));
#if QT_CONFIG(tooltip)
        CompletePathCheckBox->setToolTip(QCoreApplication::translate("qxgeditOptionsForm", "Whether to show the complete directory path of loaded session files", nullptr));
#endif // QT_CONFIG(tooltip)
        CompletePathCheckBox->setText(QCoreApplication::translate("qxgeditOptionsForm", "Show complete &path of session files", nullptr));
        MaxRecentFilesTextLabel->setText(QCoreApplication::translate("qxgeditOptionsForm", "&Number of recent files:", nullptr));
#if QT_CONFIG(tooltip)
        MaxRecentFilesSpinBox->setToolTip(QCoreApplication::translate("qxgeditOptionsForm", "The maximum number of recent files to keep in menu", nullptr));
#endif // QT_CONFIG(tooltip)
        RandomizePercentTextLabel->setText(QCoreApplication::translate("qxgeditOptionsForm", "&Randomize factor:", nullptr));
#if QT_CONFIG(tooltip)
        RandomizePercentSpinBox->setToolTip(QCoreApplication::translate("qxgeditOptionsForm", "Randomizer deviation from current value", nullptr));
#endif // QT_CONFIG(tooltip)
        RandomizePercentSpinBox->setSuffix(QCoreApplication::translate("qxgeditOptionsForm", " %", nullptr));
        BaseFontSizeTextLabel->setText(QCoreApplication::translate("qxgeditOptionsForm", "&Base font size:", nullptr));
        BaseFontSizeComboBox->setItemText(0, QCoreApplication::translate("qxgeditOptionsForm", "(default)", nullptr));
        BaseFontSizeComboBox->setItemText(1, QCoreApplication::translate("qxgeditOptionsForm", "6", nullptr));
        BaseFontSizeComboBox->setItemText(2, QCoreApplication::translate("qxgeditOptionsForm", "7", nullptr));
        BaseFontSizeComboBox->setItemText(3, QCoreApplication::translate("qxgeditOptionsForm", "8", nullptr));
        BaseFontSizeComboBox->setItemText(4, QCoreApplication::translate("qxgeditOptionsForm", "9", nullptr));
        BaseFontSizeComboBox->setItemText(5, QCoreApplication::translate("qxgeditOptionsForm", "10", nullptr));
        BaseFontSizeComboBox->setItemText(6, QCoreApplication::translate("qxgeditOptionsForm", "11", nullptr));
        BaseFontSizeComboBox->setItemText(7, QCoreApplication::translate("qxgeditOptionsForm", "12", nullptr));

#if QT_CONFIG(tooltip)
        BaseFontSizeComboBox->setToolTip(QCoreApplication::translate("qxgeditOptionsForm", "Base application font size (pt.)", nullptr));
#endif // QT_CONFIG(tooltip)
        StyleThemeTextLabel->setText(QCoreApplication::translate("qxgeditOptionsForm", "&Widget style theme:", nullptr));
        StyleThemeComboBox->setItemText(0, QCoreApplication::translate("qxgeditOptionsForm", "(default)", nullptr));

#if QT_CONFIG(tooltip)
        StyleThemeComboBox->setToolTip(QCoreApplication::translate("qxgeditOptionsForm", "Custom widget style theme", nullptr));
#endif // QT_CONFIG(tooltip)
        ColorThemeTextLabel->setText(QCoreApplication::translate("qxgeditOptionsForm", "&Color palette theme:", nullptr));
        ColorThemeComboBox->setItemText(0, QCoreApplication::translate("qxgeditOptionsForm", "(default)", nullptr));
        ColorThemeComboBox->setItemText(1, QCoreApplication::translate("qxgeditOptionsForm", "Wonton Soup", nullptr));
        ColorThemeComboBox->setItemText(2, QCoreApplication::translate("qxgeditOptionsForm", "KXStudio", nullptr));

#if QT_CONFIG(tooltip)
        ColorThemeComboBox->setToolTip(QCoreApplication::translate("qxgeditOptionsForm", "Custom color palette theme", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        ColorThemeToolButton->setToolTip(QCoreApplication::translate("qxgeditOptionsForm", "Manage custom color palette themes", nullptr));
#endif // QT_CONFIG(tooltip)
        ColorThemeToolButton->setText(QCoreApplication::translate("qxgeditOptionsForm", "...", nullptr));
        OptionsTabWidget->setTabText(OptionsTabWidget->indexOf(GeneralTabPage), QCoreApplication::translate("qxgeditOptionsForm", "&General", nullptr));
        MidiInputLabel->setText(QCoreApplication::translate("qxgeditOptionsForm", "&Inputs:", nullptr));
#if QT_CONFIG(tooltip)
        MidiInputListView->setToolTip(QCoreApplication::translate("qxgeditOptionsForm", "MIDI Readable clients / Output ports", nullptr));
#endif // QT_CONFIG(tooltip)
        MidiOutputLabel->setText(QCoreApplication::translate("qxgeditOptionsForm", "&Outputs:", nullptr));
#if QT_CONFIG(tooltip)
        MidiOutputListView->setToolTip(QCoreApplication::translate("qxgeditOptionsForm", "MIDI Writable clients / Input ports", nullptr));
#endif // QT_CONFIG(tooltip)
        OptionsTabWidget->setTabText(OptionsTabWidget->indexOf(MidiTabPage), QCoreApplication::translate("qxgeditOptionsForm", "&MIDI", nullptr));
    } // retranslateUi

};

namespace Ui {
    class qxgeditOptionsForm: public Ui_qxgeditOptionsForm {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_QXGEDITOPTIONSFORM_H
