/****************************************************************************
** Meta object code from reading C++ file 'qxgeditMainForm.h'
**
** Created by: The Qt Meta Object Compiler version 68 (Qt 6.8.3)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../qxgedit-1.0.1/src/qxgeditMainForm.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qxgeditMainForm.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 68
#error "This file was generated using the moc from 6.8.3. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN15qxgeditMainFormE_t {};
} // unnamed namespace


#ifdef QT_MOC_HAS_STRINGDATA
static constexpr auto qt_meta_stringdata_ZN15qxgeditMainFormE = QtMocHelpers::stringData(
    "qxgeditMainForm",
    "contentsChanged",
    "",
    "fileNew",
    "fileOpen",
    "fileOpenRecent",
    "fileSave",
    "fileSaveAs",
    "fileExit",
    "viewMenubar",
    "bOn",
    "viewStatusbar",
    "viewToolbar",
    "viewRandomize",
    "viewOptions",
    "helpAbout",
    "helpAboutQt",
    "stabilizeForm",
    "updateRecentFilesMenu",
    "masterResetButtonClicked",
    "reverbResetButtonClicked",
    "chorusResetButtonClicked",
    "variationResetButtonClicked",
    "multipartResetButtonClicked",
    "multipartComboActivated",
    "multipartVoiceComboActivated",
    "multipartVoiceChanged",
    "multipartPartModeChanged",
    "drumsetupResetButtonClicked",
    "drumsetupComboActivated",
    "drumsetupVoiceComboActivated",
    "drumsetupNoteComboActivated",
    "uservoiceResetButtonClicked",
    "uservoiceComboActivated",
    "uservoiceElementComboActivated",
    "uservoiceSendButtonClicked",
    "uservoiceAutoSendCheckToggled",
    "uservoiceElementChanged",
    "uservoiceLoadPresetFile",
    "uservoiceSavePresetFile",
    "rpnReceived",
    "nrpnReceived",
    "sysexReceived",
    "handle_sigusr1",
    "handle_sigterm"
);
#else  // !QT_MOC_HAS_STRINGDATA
#error "qtmochelpers.h not found or too old."
#endif // !QT_MOC_HAS_STRINGDATA

Q_CONSTINIT static const uint qt_meta_data_ZN15qxgeditMainFormE[] = {

 // content:
      12,       // revision
       0,       // classname
       0,    0, // classinfo
      42,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags, initial metatype offsets
       1,    0,  266,    2, 0x0a,    1 /* Public */,
       3,    0,  267,    2, 0x09,    2 /* Protected */,
       4,    0,  268,    2, 0x09,    3 /* Protected */,
       5,    0,  269,    2, 0x09,    4 /* Protected */,
       6,    0,  270,    2, 0x09,    5 /* Protected */,
       7,    0,  271,    2, 0x09,    6 /* Protected */,
       8,    0,  272,    2, 0x09,    7 /* Protected */,
       9,    1,  273,    2, 0x09,    8 /* Protected */,
      11,    1,  276,    2, 0x09,   10 /* Protected */,
      12,    1,  279,    2, 0x09,   12 /* Protected */,
      13,    0,  282,    2, 0x09,   14 /* Protected */,
      14,    0,  283,    2, 0x09,   15 /* Protected */,
      15,    0,  284,    2, 0x09,   16 /* Protected */,
      16,    0,  285,    2, 0x09,   17 /* Protected */,
      17,    0,  286,    2, 0x09,   18 /* Protected */,
      18,    0,  287,    2, 0x09,   19 /* Protected */,
      19,    0,  288,    2, 0x09,   20 /* Protected */,
      20,    0,  289,    2, 0x09,   21 /* Protected */,
      21,    0,  290,    2, 0x09,   22 /* Protected */,
      22,    0,  291,    2, 0x09,   23 /* Protected */,
      23,    0,  292,    2, 0x09,   24 /* Protected */,
      24,    1,  293,    2, 0x09,   25 /* Protected */,
      25,    1,  296,    2, 0x09,   27 /* Protected */,
      26,    0,  299,    2, 0x09,   29 /* Protected */,
      27,    1,  300,    2, 0x09,   30 /* Protected */,
      28,    0,  303,    2, 0x09,   32 /* Protected */,
      29,    1,  304,    2, 0x09,   33 /* Protected */,
      30,    1,  307,    2, 0x09,   35 /* Protected */,
      31,    1,  310,    2, 0x09,   37 /* Protected */,
      32,    0,  313,    2, 0x09,   39 /* Protected */,
      33,    1,  314,    2, 0x09,   40 /* Protected */,
      34,    1,  317,    2, 0x09,   42 /* Protected */,
      35,    0,  320,    2, 0x09,   44 /* Protected */,
      36,    1,  321,    2, 0x09,   45 /* Protected */,
      37,    1,  324,    2, 0x09,   47 /* Protected */,
      38,    1,  327,    2, 0x09,   49 /* Protected */,
      39,    1,  330,    2, 0x09,   51 /* Protected */,
      40,    3,  333,    2, 0x09,   53 /* Protected */,
      41,    3,  340,    2, 0x09,   57 /* Protected */,
      42,    1,  347,    2, 0x09,   61 /* Protected */,
      43,    0,  350,    2, 0x09,   63 /* Protected */,
      44,    0,  351,    2, 0x09,   64 /* Protected */,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,   10,
    QMetaType::Void, QMetaType::Bool,   10,
    QMetaType::Void, QMetaType::Bool,   10,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void,
    QMetaType::Void, QMetaType::UShort,    2,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void, QMetaType::UShort,    2,
    QMetaType::Void, QMetaType::QString,    2,
    QMetaType::Void, QMetaType::QString,    2,
    QMetaType::Void, QMetaType::UChar, QMetaType::UShort, QMetaType::UShort,    2,    2,    2,
    QMetaType::Void, QMetaType::UChar, QMetaType::UShort, QMetaType::UShort,    2,    2,    2,
    QMetaType::Void, QMetaType::QByteArray,    2,
    QMetaType::Void,
    QMetaType::Void,

       0        // eod
};

Q_CONSTINIT const QMetaObject qxgeditMainForm::staticMetaObject = { {
    QMetaObject::SuperData::link<QMainWindow::staticMetaObject>(),
    qt_meta_stringdata_ZN15qxgeditMainFormE.offsetsAndSizes,
    qt_meta_data_ZN15qxgeditMainFormE,
    qt_static_metacall,
    nullptr,
    qt_incomplete_metaTypeArray<qt_meta_tag_ZN15qxgeditMainFormE_t,
        // Q_OBJECT / Q_GADGET
        QtPrivate::TypeAndForceComplete<qxgeditMainForm, std::true_type>,
        // method 'contentsChanged'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'fileNew'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'fileOpen'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'fileOpenRecent'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'fileSave'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'fileSaveAs'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'fileExit'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'viewMenubar'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<bool, std::false_type>,
        // method 'viewStatusbar'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<bool, std::false_type>,
        // method 'viewToolbar'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<bool, std::false_type>,
        // method 'viewRandomize'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'viewOptions'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'helpAbout'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'helpAboutQt'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'stabilizeForm'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'updateRecentFilesMenu'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'masterResetButtonClicked'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'reverbResetButtonClicked'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'chorusResetButtonClicked'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'variationResetButtonClicked'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'multipartResetButtonClicked'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'multipartComboActivated'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<int, std::false_type>,
        // method 'multipartVoiceComboActivated'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<int, std::false_type>,
        // method 'multipartVoiceChanged'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'multipartPartModeChanged'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<unsigned short, std::false_type>,
        // method 'drumsetupResetButtonClicked'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'drumsetupComboActivated'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<int, std::false_type>,
        // method 'drumsetupVoiceComboActivated'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<int, std::false_type>,
        // method 'drumsetupNoteComboActivated'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<int, std::false_type>,
        // method 'uservoiceResetButtonClicked'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'uservoiceComboActivated'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<int, std::false_type>,
        // method 'uservoiceElementComboActivated'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<int, std::false_type>,
        // method 'uservoiceSendButtonClicked'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'uservoiceAutoSendCheckToggled'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<bool, std::false_type>,
        // method 'uservoiceElementChanged'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<unsigned short, std::false_type>,
        // method 'uservoiceLoadPresetFile'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QString &, std::false_type>,
        // method 'uservoiceSavePresetFile'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QString &, std::false_type>,
        // method 'rpnReceived'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<unsigned char, std::false_type>,
        QtPrivate::TypeAndForceComplete<unsigned short, std::false_type>,
        QtPrivate::TypeAndForceComplete<unsigned short, std::false_type>,
        // method 'nrpnReceived'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<unsigned char, std::false_type>,
        QtPrivate::TypeAndForceComplete<unsigned short, std::false_type>,
        QtPrivate::TypeAndForceComplete<unsigned short, std::false_type>,
        // method 'sysexReceived'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<const QByteArray &, std::false_type>,
        // method 'handle_sigusr1'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'handle_sigterm'
        QtPrivate::TypeAndForceComplete<void, std::false_type>
    >,
    nullptr
} };

void qxgeditMainForm::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<qxgeditMainForm *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->contentsChanged(); break;
        case 1: _t->fileNew(); break;
        case 2: _t->fileOpen(); break;
        case 3: _t->fileOpenRecent(); break;
        case 4: _t->fileSave(); break;
        case 5: _t->fileSaveAs(); break;
        case 6: _t->fileExit(); break;
        case 7: _t->viewMenubar((*reinterpret_cast< std::add_pointer_t<bool>>(_a[1]))); break;
        case 8: _t->viewStatusbar((*reinterpret_cast< std::add_pointer_t<bool>>(_a[1]))); break;
        case 9: _t->viewToolbar((*reinterpret_cast< std::add_pointer_t<bool>>(_a[1]))); break;
        case 10: _t->viewRandomize(); break;
        case 11: _t->viewOptions(); break;
        case 12: _t->helpAbout(); break;
        case 13: _t->helpAboutQt(); break;
        case 14: _t->stabilizeForm(); break;
        case 15: _t->updateRecentFilesMenu(); break;
        case 16: _t->masterResetButtonClicked(); break;
        case 17: _t->reverbResetButtonClicked(); break;
        case 18: _t->chorusResetButtonClicked(); break;
        case 19: _t->variationResetButtonClicked(); break;
        case 20: _t->multipartResetButtonClicked(); break;
        case 21: _t->multipartComboActivated((*reinterpret_cast< std::add_pointer_t<int>>(_a[1]))); break;
        case 22: _t->multipartVoiceComboActivated((*reinterpret_cast< std::add_pointer_t<int>>(_a[1]))); break;
        case 23: _t->multipartVoiceChanged(); break;
        case 24: _t->multipartPartModeChanged((*reinterpret_cast< std::add_pointer_t<ushort>>(_a[1]))); break;
        case 25: _t->drumsetupResetButtonClicked(); break;
        case 26: _t->drumsetupComboActivated((*reinterpret_cast< std::add_pointer_t<int>>(_a[1]))); break;
        case 27: _t->drumsetupVoiceComboActivated((*reinterpret_cast< std::add_pointer_t<int>>(_a[1]))); break;
        case 28: _t->drumsetupNoteComboActivated((*reinterpret_cast< std::add_pointer_t<int>>(_a[1]))); break;
        case 29: _t->uservoiceResetButtonClicked(); break;
        case 30: _t->uservoiceComboActivated((*reinterpret_cast< std::add_pointer_t<int>>(_a[1]))); break;
        case 31: _t->uservoiceElementComboActivated((*reinterpret_cast< std::add_pointer_t<int>>(_a[1]))); break;
        case 32: _t->uservoiceSendButtonClicked(); break;
        case 33: _t->uservoiceAutoSendCheckToggled((*reinterpret_cast< std::add_pointer_t<bool>>(_a[1]))); break;
        case 34: _t->uservoiceElementChanged((*reinterpret_cast< std::add_pointer_t<ushort>>(_a[1]))); break;
        case 35: _t->uservoiceLoadPresetFile((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1]))); break;
        case 36: _t->uservoiceSavePresetFile((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1]))); break;
        case 37: _t->rpnReceived((*reinterpret_cast< std::add_pointer_t<uchar>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<ushort>>(_a[2])),(*reinterpret_cast< std::add_pointer_t<ushort>>(_a[3]))); break;
        case 38: _t->nrpnReceived((*reinterpret_cast< std::add_pointer_t<uchar>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<ushort>>(_a[2])),(*reinterpret_cast< std::add_pointer_t<ushort>>(_a[3]))); break;
        case 39: _t->sysexReceived((*reinterpret_cast< std::add_pointer_t<QByteArray>>(_a[1]))); break;
        case 40: _t->handle_sigusr1(); break;
        case 41: _t->handle_sigterm(); break;
        default: ;
        }
    }
}

const QMetaObject *qxgeditMainForm::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *qxgeditMainForm::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_ZN15qxgeditMainFormE.stringdata0))
        return static_cast<void*>(this);
    return QMainWindow::qt_metacast(_clname);
}

int qxgeditMainForm::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QMainWindow::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 42)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 42;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 42)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 42;
    }
    return _id;
}
QT_WARNING_POP
