/****************************************************************************
** Meta object code from reading C++ file 'qxgeditScale.h'
**
** Created by: The Qt Meta Object Compiler version 68 (Qt 6.8.3)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../qxgedit-1.0.1/src/qxgeditScale.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qxgeditScale.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 68
#error "This file was generated using the moc from 6.8.3. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN12qxgeditScaleE_t {};
} // unnamed namespace


#ifdef QT_MOC_HAS_STRINGDATA
static constexpr auto qt_meta_stringdata_ZN12qxgeditScaleE = QtMocHelpers::stringData(
    "qxgeditScale",
    "break1Changed",
    "",
    "iBreak1",
    "break2Changed",
    "iBreak2",
    "break3Changed",
    "iBreak3",
    "break4Changed",
    "iBreak4",
    "offset1Changed",
    "iOffset1",
    "offset2Changed",
    "iOffset2",
    "offset3Changed",
    "iOffset3",
    "offset4Changed",
    "iOffset4",
    "setBreak1",
    "setBreak2",
    "setBreak3",
    "setBreak4",
    "setOffset1",
    "setOffset2",
    "setOffset3",
    "setOffset4"
);
#else  // !QT_MOC_HAS_STRINGDATA
#error "qtmochelpers.h not found or too old."
#endif // !QT_MOC_HAS_STRINGDATA

Q_CONSTINIT static const uint qt_meta_data_ZN12qxgeditScaleE[] = {

 // content:
      12,       // revision
       0,       // classname
       0,    0, // classinfo
      16,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       8,       // signalCount

 // signals: name, argc, parameters, tag, flags, initial metatype offsets
       1,    1,  110,    2, 0x06,    1 /* Public */,
       4,    1,  113,    2, 0x06,    3 /* Public */,
       6,    1,  116,    2, 0x06,    5 /* Public */,
       8,    1,  119,    2, 0x06,    7 /* Public */,
      10,    1,  122,    2, 0x06,    9 /* Public */,
      12,    1,  125,    2, 0x06,   11 /* Public */,
      14,    1,  128,    2, 0x06,   13 /* Public */,
      16,    1,  131,    2, 0x06,   15 /* Public */,

 // slots: name, argc, parameters, tag, flags, initial metatype offsets
      18,    1,  134,    2, 0x0a,   17 /* Public */,
      19,    1,  137,    2, 0x0a,   19 /* Public */,
      20,    1,  140,    2, 0x0a,   21 /* Public */,
      21,    1,  143,    2, 0x0a,   23 /* Public */,
      22,    1,  146,    2, 0x0a,   25 /* Public */,
      23,    1,  149,    2, 0x0a,   27 /* Public */,
      24,    1,  152,    2, 0x0a,   29 /* Public */,
      25,    1,  155,    2, 0x0a,   31 /* Public */,

 // signals: parameters
    QMetaType::Void, QMetaType::UShort,    3,
    QMetaType::Void, QMetaType::UShort,    5,
    QMetaType::Void, QMetaType::UShort,    7,
    QMetaType::Void, QMetaType::UShort,    9,
    QMetaType::Void, QMetaType::UShort,   11,
    QMetaType::Void, QMetaType::UShort,   13,
    QMetaType::Void, QMetaType::UShort,   15,
    QMetaType::Void, QMetaType::UShort,   17,

 // slots: parameters
    QMetaType::Void, QMetaType::UShort,    3,
    QMetaType::Void, QMetaType::UShort,    5,
    QMetaType::Void, QMetaType::UShort,    7,
    QMetaType::Void, QMetaType::UShort,    9,
    QMetaType::Void, QMetaType::UShort,   11,
    QMetaType::Void, QMetaType::UShort,   13,
    QMetaType::Void, QMetaType::UShort,   15,
    QMetaType::Void, QMetaType::UShort,   17,

       0        // eod
};

Q_CONSTINIT const QMetaObject qxgeditScale::staticMetaObject = { {
    QMetaObject::SuperData::link<QFrame::staticMetaObject>(),
    qt_meta_stringdata_ZN12qxgeditScaleE.offsetsAndSizes,
    qt_meta_data_ZN12qxgeditScaleE,
    qt_static_metacall,
    nullptr,
    qt_incomplete_metaTypeArray<qt_meta_tag_ZN12qxgeditScaleE_t,
        // Q_OBJECT / Q_GADGET
        QtPrivate::TypeAndForceComplete<qxgeditScale, std::true_type>,
        // method 'break1Changed'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<unsigned short, std::false_type>,
        // method 'break2Changed'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<unsigned short, std::false_type>,
        // method 'break3Changed'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<unsigned short, std::false_type>,
        // method 'break4Changed'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<unsigned short, std::false_type>,
        // method 'offset1Changed'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<unsigned short, std::false_type>,
        // method 'offset2Changed'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<unsigned short, std::false_type>,
        // method 'offset3Changed'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<unsigned short, std::false_type>,
        // method 'offset4Changed'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<unsigned short, std::false_type>,
        // method 'setBreak1'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<unsigned short, std::false_type>,
        // method 'setBreak2'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<unsigned short, std::false_type>,
        // method 'setBreak3'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<unsigned short, std::false_type>,
        // method 'setBreak4'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<unsigned short, std::false_type>,
        // method 'setOffset1'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<unsigned short, std::false_type>,
        // method 'setOffset2'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<unsigned short, std::false_type>,
        // method 'setOffset3'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<unsigned short, std::false_type>,
        // method 'setOffset4'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<unsigned short, std::false_type>
    >,
    nullptr
} };

void qxgeditScale::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<qxgeditScale *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->break1Changed((*reinterpret_cast< std::add_pointer_t<ushort>>(_a[1]))); break;
        case 1: _t->break2Changed((*reinterpret_cast< std::add_pointer_t<ushort>>(_a[1]))); break;
        case 2: _t->break3Changed((*reinterpret_cast< std::add_pointer_t<ushort>>(_a[1]))); break;
        case 3: _t->break4Changed((*reinterpret_cast< std::add_pointer_t<ushort>>(_a[1]))); break;
        case 4: _t->offset1Changed((*reinterpret_cast< std::add_pointer_t<ushort>>(_a[1]))); break;
        case 5: _t->offset2Changed((*reinterpret_cast< std::add_pointer_t<ushort>>(_a[1]))); break;
        case 6: _t->offset3Changed((*reinterpret_cast< std::add_pointer_t<ushort>>(_a[1]))); break;
        case 7: _t->offset4Changed((*reinterpret_cast< std::add_pointer_t<ushort>>(_a[1]))); break;
        case 8: _t->setBreak1((*reinterpret_cast< std::add_pointer_t<ushort>>(_a[1]))); break;
        case 9: _t->setBreak2((*reinterpret_cast< std::add_pointer_t<ushort>>(_a[1]))); break;
        case 10: _t->setBreak3((*reinterpret_cast< std::add_pointer_t<ushort>>(_a[1]))); break;
        case 11: _t->setBreak4((*reinterpret_cast< std::add_pointer_t<ushort>>(_a[1]))); break;
        case 12: _t->setOffset1((*reinterpret_cast< std::add_pointer_t<ushort>>(_a[1]))); break;
        case 13: _t->setOffset2((*reinterpret_cast< std::add_pointer_t<ushort>>(_a[1]))); break;
        case 14: _t->setOffset3((*reinterpret_cast< std::add_pointer_t<ushort>>(_a[1]))); break;
        case 15: _t->setOffset4((*reinterpret_cast< std::add_pointer_t<ushort>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _q_method_type = void (qxgeditScale::*)(unsigned short );
            if (_q_method_type _q_method = &qxgeditScale::break1Changed; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 0;
                return;
            }
        }
        {
            using _q_method_type = void (qxgeditScale::*)(unsigned short );
            if (_q_method_type _q_method = &qxgeditScale::break2Changed; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 1;
                return;
            }
        }
        {
            using _q_method_type = void (qxgeditScale::*)(unsigned short );
            if (_q_method_type _q_method = &qxgeditScale::break3Changed; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 2;
                return;
            }
        }
        {
            using _q_method_type = void (qxgeditScale::*)(unsigned short );
            if (_q_method_type _q_method = &qxgeditScale::break4Changed; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 3;
                return;
            }
        }
        {
            using _q_method_type = void (qxgeditScale::*)(unsigned short );
            if (_q_method_type _q_method = &qxgeditScale::offset1Changed; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 4;
                return;
            }
        }
        {
            using _q_method_type = void (qxgeditScale::*)(unsigned short );
            if (_q_method_type _q_method = &qxgeditScale::offset2Changed; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 5;
                return;
            }
        }
        {
            using _q_method_type = void (qxgeditScale::*)(unsigned short );
            if (_q_method_type _q_method = &qxgeditScale::offset3Changed; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 6;
                return;
            }
        }
        {
            using _q_method_type = void (qxgeditScale::*)(unsigned short );
            if (_q_method_type _q_method = &qxgeditScale::offset4Changed; *reinterpret_cast<_q_method_type *>(_a[1]) == _q_method) {
                *result = 7;
                return;
            }
        }
    }
}

const QMetaObject *qxgeditScale::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *qxgeditScale::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_ZN12qxgeditScaleE.stringdata0))
        return static_cast<void*>(this);
    return QFrame::qt_metacast(_clname);
}

int qxgeditScale::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QFrame::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 16)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 16;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 16)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 16;
    }
    return _id;
}

// SIGNAL 0
void qxgeditScale::break1Changed(unsigned short _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void qxgeditScale::break2Changed(unsigned short _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void qxgeditScale::break3Changed(unsigned short _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}

// SIGNAL 3
void qxgeditScale::break4Changed(unsigned short _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 3, _a);
}

// SIGNAL 4
void qxgeditScale::offset1Changed(unsigned short _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 4, _a);
}

// SIGNAL 5
void qxgeditScale::offset2Changed(unsigned short _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 5, _a);
}

// SIGNAL 6
void qxgeditScale::offset3Changed(unsigned short _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 6, _a);
}

// SIGNAL 7
void qxgeditScale::offset4Changed(unsigned short _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 7, _a);
}
QT_WARNING_POP
